@@ -337,13 +337,10 @@ egg_xfont_selector_init (EggXFontSelector *selector)
 
   gtk_label_set_mnemonic_widget (GTK_LABEL (selector->family_label),
 				 selector->family_options);
-  terminal_util_set_labelled_by (selector->family_options, GTK_LABEL(selector->family_label));
   terminal_util_set_atk_name_description (selector->family_options,
                                           NULL, _("Click to choose font type"));
   gtk_label_set_mnemonic_widget (GTK_LABEL (size_label),
 				 selector->size_options);
-  terminal_util_set_labelled_by (selector->size_options,
-                                 GTK_LABEL (size_label));
   terminal_util_set_atk_name_description (selector->size_options,
                                           NULL, _("Click to choose font size"));
 
@@ -1701,10 +1698,22 @@ egg_xfont_selector_finalize (GObject *object)
 {
   GObjectClass *parent_class;
   EggXFontSelector *sel;
-  
+  EggXFontFilter *filter;
+  gint prop;
+
   sel = EGG_XFONT_SELECTOR (object);
   parent_class = g_type_class_ref (g_type_parent (G_OBJECT_TYPE (object)));
 
+  /* Clear the filter data. */
+  filter = &sel->filters[EGG_XFONT_FILTER_BASE];
+  
+  for (prop = 0; prop < EGG_NUM_FONT_PROPERTIES; prop++)
+    {
+      g_free(filter->property_filters[prop]);
+      filter->property_filters[prop] = NULL;
+      filter->property_nfilters[prop] = 0;
+    }
+  
   g_free (sel->filtered_font_index);
   g_free (sel->size_options_map);
 
@@ -1719,4 +1728,3 @@ egg_xfont_selector_finalize (GObject *object)
 
   g_type_class_unref (parent_class);
 }
-
