@@ -58,6 +58,7 @@ struct _TerminalScreenPrivate
   char *cooked_title, *cooked_icon_title;
   gboolean icon_title_set;
   char *matched_string;
+  int matched_flavor;
   char **override_command;
   GtkWidget *title_editor;
   GtkWidget *title_entry;
@@ -79,6 +80,14 @@ enum {
   LAST_SIGNAL
 };
 
+enum
+{
+  FLAVOR_AS_IS = 0,
+  FLAVOR_DEFAULT_TO_HTTP,
+  FLAVOR_MAILTO,
+  FLAVOR_SKEY
+};
+
 static void terminal_screen_init        (TerminalScreen      *screen);
 static void terminal_screen_class_init  (TerminalScreenClass *klass);
 static void terminal_screen_finalize    (GObject             *object);
@@ -233,17 +242,20 @@ terminal_screen_init (TerminalScreen *screen)
 #define USERCHARS "-A-Za-z0-9"
 #define PASSCHARS "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
 #define HOSTCHARS "-A-Za-z0-9"
-#define SCHEME    "(news|telnet|nttp|file|http|ftp|https)"
+#define SCHEME    "(news|telnet|nttp|file|https?|ftps?|webcal)"
 #define USER      "[" USERCHARS "]+(:["PASSCHARS "]+)?"
 
   terminal_widget_match_add (screen->priv->term,
-      "((" SCHEME "://(" USER "@)?)|(www|ftp)[" HOSTCHARS "]*\\.)[" HOSTCHARS ".]+(:[0-9]*)?");
-  
+      "\\<(" SCHEME "://(" USER "@)?)[" HOSTCHARS ".]+(:[0-9]+)?"
+      "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?\\>", FLAVOR_AS_IS);
+
   terminal_widget_match_add (screen->priv->term,
-      "((" SCHEME "://(" USER "@)?)"  "|"  "(www|ftp)[" HOSTCHARS "]*\\.)"
-      "[" HOSTCHARS ".]+(:[0-9]+)?/"
-      "[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*"
-      "[^]'.}>) \t\r\n,\\\"]");
+      "\\<(www|ftp)[" HOSTCHARS "]*\\.[" HOSTCHARS ".]+(:[0-9]+)?"
+      "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?\\>", FLAVOR_DEFAULT_TO_HTTP);
+
+  terminal_widget_match_add (screen->priv->term, 
+      
+      "\\<(mailto:)?[a-z0-9]+@[a-z0-9][a-z0-9-]*(\\.[a-z0-9][a-z0-9-]*)+\\>", FLAVOR_MAILTO);
 
   terminal_screen_setup_dnd (screen);
   
@@ -522,7 +534,7 @@ reread_profile (TerminalScreen *screen)
   if (terminal_profile_get_use_skey (screen->priv->profile))
     {
       terminal_widget_skey_match_add (screen->priv->term,
-				      "s/key [0-9]* [-A-Za-z0-9]*");
+				      "s/key [0-9]* [-A-Za-z0-9]*", FLAVOR_SKEY);
     }
   else
     {
@@ -1240,7 +1252,8 @@ secure_keyboard_callback (GtkWidget      *menu_item,
 
 static void
 open_url (TerminalScreen *screen,
-          const char     *orig_url)
+          const char     *orig_url,
+          int             flavor)
 {
   GError *err;
   char *url;
@@ -1248,9 +1261,33 @@ open_url (TerminalScreen *screen,
   g_return_if_fail (orig_url != NULL);
 
   /* this is to handle gnome_url_show reentrancy */
-  url = gnome_vfs_make_uri_from_input (orig_url);
   g_object_ref (G_OBJECT (screen));
   
+  switch (flavor)
+    {
+    case FLAVOR_DEFAULT_TO_HTTP:
+      url = g_strdup_printf ("http:%s", orig_url);
+      break;
+    case FLAVOR_MAILTO:
+      if (strncmp ("mailto:", orig_url, 7) != 0)
+        {
+          url = g_strdup_printf ("mailto:%s", orig_url);
+        }
+      else
+        {
+          url = g_strdup (orig_url);
+        }
+      break;
+    case FLAVOR_AS_IS:
+    case FLAVOR_SKEY:
+      url = g_strdup (orig_url);
+      break;
+    default:
+      url = NULL;
+      g_assert_not_reached ();
+    }
+
+  g_printerr ("(%s)(%s)\n", orig_url, url);
   err = NULL;
   gnome_url_show (url, &err);
 
@@ -1281,7 +1318,7 @@ open_url_callback (GtkWidget      *menu_item,
 {
   if (screen->priv->matched_string)
     {
-      open_url (screen, screen->priv->matched_string);
+      open_url (screen, screen->priv->matched_string, screen->priv->matched_flavor);
       g_free (screen->priv->matched_string);
       screen->priv->matched_string = NULL;
     }
@@ -1411,6 +1448,7 @@ terminal_screen_do_popup (TerminalScreen *screen,
   GList *profiles;
   GList *tmp;
   GSList *group;
+  GtkClipboard *clip;
   
   if (screen->priv->popup_menu)
     gtk_widget_destroy (screen->priv->popup_menu);
@@ -1457,10 +1495,16 @@ terminal_screen_do_popup (TerminalScreen *screen,
   gtk_widget_show (menu_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (screen->priv->popup_menu), menu_item);
 
-  menu_item = append_menuitem (screen->priv->popup_menu,
-                               _("C_lose Tab"),
-                               G_CALLBACK (close_tab_callback),
-                               screen);
+  if (terminal_window_get_screen_count (terminal_screen_get_window (screen)) > 1)
+    menu_item = append_menuitem (screen->priv->popup_menu,
+                                 _("C_lose Tab"),
+                                 G_CALLBACK (close_tab_callback),
+                                 screen);
+  else
+    menu_item = append_menuitem (screen->priv->popup_menu,
+                                 _("_Close Window"),
+                                 G_CALLBACK (close_tab_callback),
+                                 screen);
 
   menu_item = gtk_separator_menu_item_new ();
   gtk_widget_show (menu_item);
@@ -1478,6 +1522,8 @@ terminal_screen_do_popup (TerminalScreen *screen,
                                      GTK_STOCK_PASTE,
                                      G_CALLBACK (paste_callback),
                                      screen);
+  clip = gtk_clipboard_get (GDK_NONE);
+  gtk_widget_set_sensitive (menu_item, gtk_clipboard_wait_is_text_available (clip));
   
   menu_item = gtk_separator_menu_item_new ();
   gtk_widget_show (menu_item);
@@ -1602,7 +1648,8 @@ terminal_screen_button_press_event (GtkWidget      *widget,
   screen->priv->matched_string =
     terminal_widget_check_match (term,
                                  event->x / char_width,
-                                 event->y / char_height);
+                                 event->y / char_height,
+                                 &screen->priv->matched_flavor);
   dingus_button = ((event->button == 1) || (event->button == 2));
 
   if (dingus_button &&
@@ -1613,7 +1660,8 @@ terminal_screen_button_press_event (GtkWidget      *widget,
 
       skey_match = terminal_widget_skey_check_match (term,
 						     event->x / char_width,
-						     event->y / char_height);
+						     event->y / char_height,
+                                                     NULL);
       if (skey_match != NULL)
 	{
 	  terminal_skey_do_popup (screen, GTK_WINDOW (terminal_screen_get_window (screen)), skey_match);
@@ -1629,7 +1677,7 @@ terminal_screen_button_press_event (GtkWidget      *widget,
     {
       gtk_widget_grab_focus (widget);
       
-      open_url (screen, screen->priv->matched_string);
+      open_url (screen, screen->priv->matched_string, screen->priv->matched_flavor);
       g_free (screen->priv->matched_string);
       screen->priv->matched_string = NULL;
       return TRUE; /* don't do anything else such as select with the click */
@@ -1869,6 +1917,8 @@ terminal_screen_widget_child_died (GtkWidget      *term,
     case TERMINAL_EXIT_RESTART:
       terminal_screen_launch_child (screen);
       break;
+    case TERMINAL_EXIT_HOLD:
+      break;
     }
 }
 
