@@ -98,6 +98,10 @@ static void terminal_window_class_init  (TerminalWindowClass *klass);
 static void terminal_window_finalize    (GObject             *object);
 static void terminal_window_destroy     (GtkObject           *object);
 
+static gboolean terminal_window_delete_event (GtkWidget *widget,
+                                              GdkEvent *event,
+                                              gpointer data);
+
 static void       screen_set_scrollbar (TerminalScreen *screen,
                                         GtkWidget      *scrollbar);
 static void       screen_set_hbox      (TerminalScreen *screen,
@@ -206,6 +210,8 @@ static gboolean find_smaller_zoom_factor (double  current,
 
 static void terminal_window_show (GtkWidget *widget);
 
+static gboolean confirm_close_window (TerminalWindow *window);
+
 static gpointer parent_class;
 
 GType
@@ -748,7 +754,10 @@ terminal_window_init (TerminalWindow *window)
   GError *error;
   gboolean menus_have_icons;
 
-  
+  g_signal_connect (G_OBJECT (window), "delete_event", 
+                    G_CALLBACK(terminal_window_delete_event),
+                    NULL);
+
   gtk_window_set_title (GTK_WINDOW (window), _("Terminal"));
   client = gconf_client_get_default();
   
@@ -757,7 +766,6 @@ terminal_window_init (TerminalWindow *window)
   window->priv->active_term = NULL;
   window->priv->menubar = gtk_menu_bar_new ();
   window->priv->menubar_visible = FALSE;
-  g_object_ref (G_OBJECT (window->priv->menubar)); /* so we can add/remove */
   
   window->priv->main_vbox = gtk_vbox_new (FALSE, 0);
   window->priv->notebook = gtk_notebook_new ();
@@ -797,6 +805,10 @@ terminal_window_init (TerminalWindow *window)
   gtk_container_add (GTK_CONTAINER (window),
                      window->priv->main_vbox);
 
+  gtk_box_pack_start (GTK_BOX (window->priv->main_vbox),
+		      window->priv->menubar,
+		      FALSE, FALSE, 0);
+
   gtk_box_pack_end (GTK_BOX (window->priv->main_vbox),
                     window->priv->notebook,
                     TRUE, TRUE, 0);  
@@ -896,7 +908,7 @@ terminal_window_init (TerminalWindow *window)
   if (!gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE)))
     gtk_widget_set_sensitive (mi, FALSE);
   else
-    g_signal_connect (G_OBJECT (window), "window-state-event", G_CALLBACK (window_state_event_callback), NULL);
+    g_signal_connect (G_OBJECT (window), "window-state-event", G_CALLBACK (window_state_event_callback), mi);
 
   mi = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
@@ -1067,9 +1079,6 @@ terminal_window_destroy (GtkObject *object)
   while (window->priv->terms)
     terminal_window_remove_screen (window, window->priv->terms->data);
 
-  if (window->priv->menubar)
-    g_object_unref (G_OBJECT (window->priv->menubar));
-
   g_list_free (window->priv->tab_menuitems);
   window->priv->tab_menuitems = NULL;
   
@@ -1086,6 +1095,14 @@ terminal_window_destroy (GtkObject *object)
   GTK_OBJECT_CLASS (parent_class)->destroy (object);  
 }
 
+static gboolean
+terminal_window_delete_event (GtkWidget *widget,
+                              GdkEvent *event,
+                              gpointer data)
+{
+   return !confirm_close_window (TERMINAL_WINDOW (widget));
+}
+
 static void
 sn_error_trap_push (SnDisplay *display,
                     Display   *xdisplay)
@@ -1263,14 +1280,20 @@ icon_title_changed_callback (TerminalScreen *screen,
 static void
 update_copy_sensitivity (TerminalWindow *window)
 {
+  GtkClipboard *clip = gtk_clipboard_get (GDK_NONE);
   gboolean can_copy = FALSE;
+  gboolean can_paste = FALSE;
 
   if (window->priv->active_term)
-    can_copy = terminal_screen_get_text_selected (window->priv->active_term);
+    {
+      can_copy = terminal_screen_get_text_selected (window->priv->active_term);
+      can_paste = gtk_clipboard_wait_is_text_available (clip);
+    }
   else
-    can_copy = FALSE;
+    can_copy = can_paste = FALSE;
 
   gtk_widget_set_sensitive (window->priv->copy_menuitem, can_copy);
+  gtk_widget_set_sensitive (window->priv->paste_menuitem, can_paste);
 }
 
 static void
@@ -1511,6 +1534,12 @@ terminal_window_list_screens (TerminalWindow *window)
   return g_list_copy (window->priv->terms);
 }
 
+int    
+terminal_window_get_screen_count (TerminalWindow *window)
+{
+  return g_list_length (window->priv->terms);
+}
+
 void
 terminal_window_set_menubar_visible (TerminalWindow *window,
                                      gboolean        setting)
@@ -1528,15 +1557,11 @@ terminal_window_set_menubar_visible (TerminalWindow *window,
   
   if (window->priv->menubar_visible)
     {      
-      gtk_box_pack_start (GTK_BOX (window->priv->main_vbox),
-                          window->priv->menubar,
-                          FALSE, FALSE, 0);
       gtk_widget_show (window->priv->menubar);
     }
   else
-    {      
-      gtk_container_remove (GTK_CONTAINER (window->priv->main_vbox),
-                            window->priv->menubar);
+    {
+      gtk_widget_hide (window->priv->menubar);
     }
 
   if (window->priv->active_term)
@@ -1668,8 +1693,6 @@ terminal_window_set_active (TerminalWindow *window,
   gdk_window_set_icon_name (GTK_WIDGET (window)->window, terminal_screen_get_icon_title (screen));
   gtk_window_set_title (GTK_WINDOW (window), terminal_screen_get_title (screen));
 
-  update_copy_sensitivity (window);
-  
   gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook),
                                  gtk_notebook_page_num (GTK_NOTEBOOK (window->priv->notebook),
                                                         screen_get_hbox (screen)));
@@ -1680,6 +1703,8 @@ terminal_window_set_active (TerminalWindow *window,
 #endif
   terminal_window_set_size (window, screen, TRUE);
   
+  update_copy_sensitivity (window);
+  
   fill_in_config_picker_submenu (window);
   fill_in_new_term_submenus (window);
   update_zoom_items (window);
@@ -2244,11 +2269,56 @@ new_tab_callback (GtkWidget      *menuitem,
     }
 }
 
+static gboolean
+confirm_close_window (TerminalWindow *window)
+{
+  GtkWidget *dialog;
+  GError *error;
+  gboolean result;
+  char *msg, *msg1;
+  int n;
+
+  n = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->priv->notebook));
+  if (n <= 1)
+    return TRUE;
+
+  error = NULL;
+  if (!gconf_client_get_bool (window->priv->conf, CONF_GLOBAL_PREFIX "/confirm_window_close", &error))
+    return TRUE;
+
+  msg1 = g_strdup_printf (ngettext ("This window has one tab open. Closing the window will close it.",
+                                    "This window has %d tabs open. Closing the window will also close all its tabs.",
+                                    n),
+                          n);
+  msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n", 
+                         _("Close all tabs?"), 
+                         msg1);
+
+  dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (window),
+                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_WARNING,
+                                   GTK_BUTTONS_NONE,
+                                   msg);
+  g_free (msg);
+  g_free (msg1);
+
+  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), _("Close All Tabs"), GTK_RESPONSE_YES);
+
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
+
+  result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
+  gtk_widget_destroy (dialog);
+
+  return result;
+}
+
 static void
 close_window_callback (GtkWidget      *menuitem,
                        TerminalWindow *window)
 {
-  gtk_widget_destroy (GTK_WIDGET (window));
+  if (confirm_close_window (window))
+    gtk_widget_destroy (GTK_WIDGET (window));
 }
 
 static void
@@ -2313,7 +2383,11 @@ change_configuration_callback (GtkWidget      *menu_item,
   g_assert (profile);
 
   if (!terminal_profile_get_forgotten (profile))
-    terminal_screen_set_profile (window->priv->active_term, profile);
+    {
+      g_signal_handlers_block_by_func (G_OBJECT (window->priv->active_term), G_CALLBACK (profile_set_callback), window);
+      terminal_screen_set_profile (window->priv->active_term, profile);
+      g_signal_handlers_unblock_by_func (G_OBJECT (window->priv->active_term), G_CALLBACK (profile_set_callback), window);
+    }
 }
 
 static void
